function EIDOLON=remask_eidolon(Diff_scaleim,RANDF_scale,DISARRAY_MASK)

% usage: EIDOLON=remask_eidolon(RANDF_scale,Diff_scaleim,DISARRAY_MASK)
% 
% Produces eidolon given the disarrayed image, the noise field and a mask. Allows to differently mask the same eidolon
% 
% INPUTS
% Diff_scaleim -> Scale-decomposed image
% RANDF_scale -> Scale-decomposed noise field
% DISARRAY_MASK -> (optional) Masks disarray fields. Must be a matrix same size
% %   as image_matrix, with values between 0 and 1.The values are multiplied
% %   with the disarray fields, so that 1 means full disarray and 0 no
% %   disarray.
% 
% OUTPUT:
% %
% % EIDOLON: Has the same format as the image_matrix (color or grayscale image);

IMSIZE=size(Diff_scaleim{1});
NOISESIZE=size(RANDF_scale{1});
if (any(~(size(Diff_scaleim)==size(RANDF_scale))))
    error('Diff_scaleim and RANDF_scale must have the same number of scales')
elseif any(NOISESIZE(1:2)~=IMSIZE(1:2))
    error('Diff_scaleim and RANDF_scale must have the same image size')
end

% Check masking field format
mask_disarray_field=0;
if ((nargin>=3)&(~isempty(DISARRAY_MASK)))
    if ((any(DISARRAY_MASK(:))>1) | (any(DISARRAY_MASK(:))<0))
        error('Disarray mask values need to be between 0 and 1');
    end
    if length(size(DISARRAY_MASK))>2
        error('DISARRAY_MASK should be NxM with as many rows and columns as image_matrix');
    elseif any(size(DISARRAY_MASK)~=IMSIZE(1:2))
        error('DISARRAY_MASK should be NxM with as many rows and columns as image_matrix');
    end
    mask_disarray_field=1;
end
    
    

if mask_disarray_field
    disp('Mask Disarray Fields');
    tic
    for nscale=1:length(RANDF_scale)
        for coorxy=1:2
            RANDF_scale{nscale}(:,:,coorxy)=RANDF_scale{nscale}(:,:,coorxy).*DISARRAY_MASK;
        end
    end
    disp(['Done ',num2str(toc),' s']);
end


disp('Apply Disarray');
tic
EIDOLON_scale=Apply_Disarray(RANDF_scale,Diff_scaleim);
disp(['Done ',num2str( toc),' s']);

disp('Assemble Eidolon');
tic
EIDOLON=Assemble_Eidolon(EIDOLON_scale);
disp(['Done ',num2str( toc),' s']);



